using System;
using System.Collections.Generic;
using System.Linq;

namespace Protogame
{
    class DefaultPlatforming : IPlatforming
    {
        private IBoundingBoxUtilities m_BoundingBoxUtilities;
        
        public DefaultPlatforming(IBoundingBoxUtilities boundingBoxUtilities)
        {
            this.m_BoundingBoxUtilities = boundingBoxUtilities;
        }
    
        public void PerformHorizontalAlignment(IBoundingBox entity, int cellWidth, int cellAlignment, int maxAdjust, Action simulateLeft, Action simulateRight)
        {
            // Perform cell alignment.
            float rem = entity.X % cellWidth;
            if (rem > cellWidth - cellAlignment ||
                rem < cellAlignment)
            {
                float targetX = (float)Math.Round(entity.X / cellWidth) * cellWidth;
                if (Math.Abs(targetX - entity.X) > maxAdjust)
                {
                    if (targetX > entity.X)
                        simulateRight();
                    else if (targetX < entity.X)
                        simulateLeft();
                }
                else
                    entity.X = targetX;
            }
        }
        
        public bool IsOnGround(IBoundingBox entity, IEnumerable<IBoundingBox> entities, Func<IBoundingBox, bool> ground)
        {
            var entityExtended = new BoundingBox
            {
                X = entity.X,
                Y = entity.Y + 1,
                Width = entity.Width,
                Height = entity.Height,
                XSpeed = entity.XSpeed,
                YSpeed = entity.YSpeed
            };
            var collidableEntities = entities.Where(ground).Where(x => x != entity).ToArray();
            foreach (var collidableEntity in collidableEntities)
            {
                if (this.m_BoundingBoxUtilities.Overlaps(entityExtended, collidableEntity))
                    return true;
            }
            return false;
        }
        
        public void ApplyActionUntil(IBoundingBox entity, Action<IBoundingBox> action, Func<IBoundingBox, bool> check, int? maximum)
        {
            if (maximum == null)
                while (!check(entity))
                    action(entity);
            else
                for (var i = 0; i < maximum.Value; i++)
                    if (!check(entity))
                        action(entity);
        }
        
        public void ApplyGravity(IHasVelocity entity, float xGravity, float yGravity)
        {
            entity.XSpeed += xGravity;
            entity.YSpeed += yGravity;
        }
        
        public void ClampSpeed(IHasVelocity entity, float? maximumXSpeed, float? maximumYSpeed)
        {
            if (maximumXSpeed != null)
            {
                if (entity.XSpeed < -Math.Abs(maximumXSpeed.Value))
                    entity.XSpeed = -Math.Abs(maximumXSpeed.Value);
                if (entity.XSpeed > Math.Abs(maximumXSpeed.Value))
                    entity.XSpeed = Math.Abs(maximumXSpeed.Value);
            }
            if (maximumYSpeed != null)
            {
                if (entity.YSpeed < -Math.Abs(maximumYSpeed.Value))
                    entity.YSpeed = -Math.Abs(maximumYSpeed.Value);
                if (entity.YSpeed > Math.Abs(maximumYSpeed.Value))
                    entity.YSpeed = Math.Abs(maximumYSpeed.Value);
            }
        }
        
        public void ApplyMovement(IBoundingBox entity, int xAmount, int yAmount, IEnumerable<IBoundingBox> entities, Func<IBoundingBox, bool> ground)
        {
            var collidableEntities = entities.Where(ground).ToArray();
            for (var x = 0; x < Math.Abs(xAmount); x++)
            {
                entity.X += Math.Sign(xAmount);
                foreach (var other in collidableEntities)
                    if (this.m_BoundingBoxUtilities.Overlaps(entity, other))
                    {
                        entity.X -= Math.Sign(xAmount);
                        goto endX;
                    }
            }
            endX:
            for (var y = 0; y < Math.Abs(yAmount); y++)
            {
                entity.Y += Math.Sign(yAmount);
                foreach (var other in collidableEntities)
                    if (this.m_BoundingBoxUtilities.Overlaps(entity, other))
                    {
                        entity.Y -= Math.Sign(yAmount);
                        goto endY;
                    }
            }
            endY:
            return;
        }
    }
}

